<?php

namespace Drupal\az_blob_fs;

use MicrosoftAzure\Storage\Blob\BlobRestProxy;
use MicrosoftAzure\Storage\Common\Exceptions\ServiceException;
use MicrosoftAzure\Storage\Common\Internal\Authentication\SharedAccessSignatureAuthScheme;
use MicrosoftAzure\Storage\Common\Internal\Authentication\SharedKeyAuthScheme;
use MicrosoftAzure\Storage\Common\Internal\Middlewares\CommonRequestMiddleware;
use MicrosoftAzure\Storage\Blob\Internal\BlobResources as Resources;
use MicrosoftAzure\Storage\Common\Internal\StorageServiceSettings;
use MicrosoftAzure\Storage\Common\Internal\Utilities;
use function watchdog_exception;

/**
 * Class Azure Blob Rest Proxy Alter.
 */
class AzBlobRestProxyAlter extends BlobRestProxy {

  /**
   * Renames a blob. Actually makes a copy of it and removes the old one.
   *
   * @param mixed $source_container
   *   The source container.
   * @param mixed $source_name
   *   The source name.
   * @param mixed $destination_container
   *   The destination container.
   * @param mixed $destination_name
   *   The destination name.
   *
   * @return bool
   *   Returns TRUE if the operation was successful, FALSE otherwise.
   */
  public function renameBlob($source_container, $source_name, $destination_container, $destination_name): bool {
    try {
      $this->copyBlob($destination_container, $destination_name, $source_container, $source_name);
      $this->deleteBlob($source_container, $source_name);
    }
    catch (ServiceException $e) {
      watchdog_exception('Azure Blob Rest Proxy Alter Exception (renameBlob)', $e);
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Indicates if provided uri is a file.
   *
   * @param string $uri
   *   The uri to validate.
   *
   * @return bool
   *   Returns TRUE if the uri is a file, FALSE otherwise.
   */
  public function uriIsFile($uri): bool {
    $parts = explode('/', $uri);
    $file_name = end($parts);
    return stripos($file_name, '.') !== FALSE;
  }

  /**
   * Get prefixed blob.
   *
   * @param mixed $container
   *   The container.
   * @param mixed $uri
   *   The container.
   *
   * @return false|\MicrosoftAzure\Storage\Blob\Models\GetBlobResult
   *   Returns the blob results.
   */
  public function getPrefixedBlob($container, $uri) {
    try {
      return $this->getBlob($container, $uri);
    }
    catch (ServiceException $e) {
      return FALSE;
    }
  }

  /**
   * Create blob service wrapper.
   *
   * @param string $connectionString
   *   The connection string.
   * @param array $options
   *   The options.
   *
   * @return \Drupal\az_blob_fs\AzBlobRestProxyAlter
   *   Returns the  Azure Blob Rest Proxy Alter.
   */
  public static function createBlobService($connectionString, array $options = []): AzBlobRestProxyAlter {
    $settings = StorageServiceSettings::createFromConnectionString(
      $connectionString
    );

    $primaryUri = Utilities::tryAddUrlScheme(
      $settings->getBlobEndpointUri()
    );

    $secondaryUri = Utilities::tryAddUrlScheme(
      $settings->getBlobSecondaryEndpointUri()
    );

    $blobWrapper = new AzBlobRestProxyAlter(
      $primaryUri,
      $secondaryUri,
      $settings->getName(),
      $options
    );

    // Getting authentication scheme.
    if ($settings->hasSasToken()) {
      $authScheme = new SharedAccessSignatureAuthScheme(
        $settings->getSasToken()
      );
    }
    else {
      $authScheme = new SharedKeyAuthScheme(
        $settings->getName(),
        $settings->getKey()
      );
    }

    // Adding common request middleware.
    $commonRequestMiddleware = new CommonRequestMiddleware(
      $authScheme,
      Resources::STORAGE_API_LATEST_VERSION,
      Resources::BLOB_SDK_VERSION
    );
    $blobWrapper->pushMiddleware($commonRequestMiddleware);

    return $blobWrapper;
  }

}
